"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handleRefAssignment = handleRefAssignment;
exports.useEnhancedReducer = exports.useKeyActionTypes = exports.targetItemPropsDefaultValue = exports.initialState = exports.getAction = void 0;

var _react = require("react");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var initialState = {
  // Pressed key
  key: null,
  isEscape: false,
  isEnter: false,
  isCapsLock: false,
  isTab: false,
  isSpace: false,
  // Function keys
  isFunctionKey: false,
  // Arrow keys
  isArrow: false,
  isArrowRight: false,
  isArrowLeft: false,
  isArrowUp: false,
  isArrowDown: false,
  // Modifier keys
  isWithShift: false,
  isWithCtrl: false,
  isWithMeta: false,
  isWithAlt: false,
  // Character varients
  isCaps: false,
  isSmall: false,
  isNumber: false,
  // For special character
  isSpecialCharacter: false
};
exports.initialState = initialState;
var useKeyActionTypes = {
  ENTER_KEY: 'ENTER_KEY',
  ESCAPE_KEY: 'ESCAPE_KEY',
  RESET_CAPTURES: 'RESET_CAPTURES',
  CAPS_ALPHABET: 'CAPS',
  SMALL_ALPHABET: 'SMALL',
  NUMBER: 'NUMBER',
  SPACE: 'SPACE',
  ARROWS: 'ARROWS',
  SPECIAL: 'SPECIAL',
  TAB: 'TAB',
  CAPSLOCK: 'CAPSLOCK',
  SHIFT: 'SHIFT'
};
exports.useKeyActionTypes = useKeyActionTypes;
var modifierKeys = {
  ctrlKey: 'Ctrl',
  shiftKey: 'Shift',
  altKey: 'Alt',
  metaKey: 'Meta'
};

var keyCodeMapper = _defineProperty({
  Enter: useKeyActionTypes.ENTER_KEY,
  Escape: useKeyActionTypes.ESCAPE_KEY,
  Tab: useKeyActionTypes.TAB,
  CapsLock: useKeyActionTypes.CAPSLOCK,
  Shift: useKeyActionTypes.SHIFT
}, ' ', useKeyActionTypes.SPACE);

var getArrowKeysPayload = function getArrowKeysPayload(key) {
  return _defineProperty({}, "is".concat(key), true);
};

var isCapitalLetterPressed = function isCapitalLetterPressed(key) {
  return /^[A-Z]$/.test(key);
};

var isSmallLetterPressed = function isSmallLetterPressed(key) {
  return /^[a-z]$/.test(key);
};

var isNumberPressed = function isNumberPressed(key) {
  return /^[0-9]/.test(key);
};

var isSpecialCharacterPressed = function isSpecialCharacterPressed(key) {
  return !isCapitalLetterPressed(key) && !isSmallLetterPressed(key) && !isNumberPressed(key) && !keyCodeMapper[key];
};

var getModifierPayload = function getModifierPayload(eventDetails) {
  var modifierPayloadObj = {};
  var modifierObjKeys = Object.keys(modifierKeys);

  for (var _i = 0, _modifierObjKeys = modifierObjKeys; _i < _modifierObjKeys.length; _i++) {
    var key = _modifierObjKeys[_i];

    if (eventDetails[key]) {
      modifierPayloadObj["isWith".concat(modifierKeys[key])] = true;
    }
  }

  return modifierPayloadObj;
};
/**
 * Returns the action type for the key pressed
 * @param {KeyboardEvent} eventDetails keyboard event object
 * @return {String}  action type
 */


var getAction = function getAction(eventDetails) {
  if (!eventDetails) {
    throw new Error('Event called with no details');
  }

  if (eventDetails.key.includes('Arrow')) {
    return {
      type: useKeyActionTypes.ARROWS,
      payload: _objectSpread({}, getArrowKeysPayload(eventDetails.key), {}, getModifierPayload(eventDetails), {
        key: eventDetails.key
      })
    };
  }

  var type;
  if (keyCodeMapper[eventDetails.key]) type = keyCodeMapper[eventDetails.key];

  if (isCapitalLetterPressed(eventDetails.key)) {
    type = useKeyActionTypes.CAPS_ALPHABET;
  }

  if (isSmallLetterPressed(eventDetails.key)) {
    type = useKeyActionTypes.SMALL_ALPHABET;
  }

  if (isNumberPressed(eventDetails.key)) {
    type = useKeyActionTypes.NUMBER;
  }

  if (!type && isSpecialCharacterPressed(eventDetails.key)) {
    type = useKeyActionTypes.SPECIAL;
  }

  if (!type) {
    type = 'SOME_OTHER_KEY';
  }

  return {
    type: type,
    payload: _objectSpread({}, getModifierPayload(eventDetails), {
      key: eventDetails.key
    })
  };
};

exports.getAction = getAction;
var targetItemPropsDefaultValue = {
  type: 'text'
};
exports.targetItemPropsDefaultValue = targetItemPropsDefaultValue;

function handleRefAssignment() {
  for (var _len = arguments.length, refs = new Array(_len), _key = 0; _key < _len; _key++) {
    refs[_key] = arguments[_key];
  }

  return function (node) {
    refs.forEach(function (ref) {
      if (typeof ref === 'function') {
        ref(node);
      } else if (ref) {
        ref.current = node;
      }
    });
  };
}

var useEnhancedReducer = function useEnhancedReducer(reducer) {
  return (0, _react.useCallback)(function (state, action) {
    return reducer(state, action);
  }, [reducer]);
};

exports.useEnhancedReducer = useEnhancedReducer;